@@ -4,6 +4,7 @@ Polynomials
 
 from math import prod
 from numbers import Number
+from random import randint
 from .primes import is_prime
 #from .factor import factorint
 
@@ -24,7 +25,7 @@ class Poly:
 
     def __init__(self, coeff: list, ring=None, modulus: list = None):
         self.coeff = list(coeff)
-        for i in range(len(self.coeff) - 1, 0, -1):
+        for i in range(len(self.coeff) - 1, 0, -1):  # strip leading zeros
             if self.coeff[i]:
                 break
             self.coeff.pop(i)
@@ -49,6 +50,9 @@ class Poly:
     def __len__(self):
         return len(self.coeff)
 
+    def __hash__(self):
+        return hash(tuple(self.coeff))
+
     def __repr__(self, latex = False):
         def prx(i: int) -> str:
             if i == 0:
@@ -142,27 +146,34 @@ class Poly:
                     ring = None
         return zero, one, ring
 
+    def _guess_field(self):
+        zero, one, field = self._guess_ring()
+        if not field:
+            raise ValueError("The polynomial does not seem to be over a known finite field.")
+        if hasattr(field, 'n'):  # Zmod
+            p = field.n
+            q = field.n
+            if not field.is_field():
+                raise ValueError("The polynomial does not seem to be over a finite field.")
+        else:  # G2 or galois
+            p = field.characteristic
+            q = field.order
+        return field, zero, one, q, p
+    
     def __add__(self, other: "Poly") -> "Poly":
-        zero = 0 * self.coeff[0]
         if not isinstance(other, self.__class__):
             if self._check_type(other):
                 tmp = self.coeff[:]
                 tmp[0] += other
                 return self.__class__(tmp, modulus=self.modulus)
             return NotImplemented
-        ls, lo = len(self.coeff), len(other.coeff)
-        if ls < lo:
-            scoeff = self.coeff + (lo - ls) * [zero]
-        else:
-            scoeff = self.coeff
-        if ls > lo:
-            ocoeff = other.coeff + (ls - lo) * [zero]
-        else:
-            ocoeff = other.coeff
         modulus = self.modulus
         if not modulus and other.modulus:
             modulus = other.modulus
-        return self.__class__([s + o for s, o in zip(scoeff, ocoeff)], modulus=modulus)
+        ls, lo = len(self.coeff), len(other.coeff)
+        if ls < lo:
+            return self.__class__([ s + o for s, o in zip(self.coeff, other.coeff) ] + other.coeff[ls:], modulus=modulus)
+        return self.__class__([ s + o for s, o in zip(self.coeff, other.coeff) ] + self.coeff[lo:], modulus=modulus)
 
     def __radd__(self, other: "Poly") -> "Poly":
         return self + other
@@ -186,9 +197,7 @@ class Poly:
             return NotImplemented
         zero = 0 * self.coeff[0]
         ls, lo = len(self.coeff), len(other.coeff)
-        coeff = [None] * (ls + lo - 1)
-        for k in range(ls + lo - 1):
-            coeff[k] = sum((self.coeff[j] * other.coeff[k - j] for j in range(max(0, k - lo + 1), min(ls, k + 1))), start=zero)
+        coeff = [ sum((self.coeff[j] * other.coeff[k - j] for j in range(max(0, k - lo + 1), min(ls, k + 1))), start=zero) for k in range(ls + lo - 1) ]
         modulus = self.modulus
         if not modulus and other.modulus:
             modulus = other.modulus
@@ -308,7 +317,7 @@ class Poly:
             tmp = self.coeff[sd - i] * one # "* 1" is here to make sure we get a copy
             for j in range(od + 1):
                 self.coeff[sd - i - j] -= tmp * oth[od - j]
-        for i in range(len(self.coeff) - 1, 0, -1):
+        for i in range(len(self.coeff) - 1, 0, -1): # strip leading zeros
             if self.coeff[i]:
                 break
             self.coeff.pop(i)
@@ -318,53 +327,60 @@ class Poly:
         if not other:
             other = self.modulus
         zero, one, ring = self._guess_ring()
-        if isinstance(other, list):
-            other = self.__class__(other, ring=ring)
-        elif not isinstance(other, self.__class__):
-            raise NotImplementedError(f"Cannot invert {self} modulo {other}.")
+        if isinstance(other, self.__class__):
+            other = other.coeff
+        elif not isinstance(other, list):
+            raise NotImplementedError(f"{other} must be a list of coefficients or a polynomial.")
+        other = Poly(other, ring = ring)
         if not other:
             raise NotImplementedError(f"{other} must be nonzero.")
         r0, r1 = other, self
-        y0, y1 = self.__class__([zero], modulus=self.modulus), self.__class__([one], modulus=self.modulus)
+        y0, y1 = self.__class__([zero]), self.__class__([one])
         while r1:
             q, r = r0.divmod(r1)
             r0, r1 = r1, r
             y0, y1 = y1, y0 - q * y1
         if r0.degree() != 0:
             raise ValueError(f"{self} is not invertible mod {other}.")
-        tmp = one / r0[0]
-        for i in range(len(y0)):
-            y0.coeff[i] *= tmp
-        return y0
+        y0.modulus = self.modulus
+        return y0 / r0.coeff[0]
 
     def gcd(self, other: "Poly") -> "Poly":
         "Greates common divisor with a given polynomial."
         ring = self._guess_ring()[-1]
-        if isinstance(other, list):
-            other = self.__class__(other, ring=ring)
-        elif not isinstance(other, self.__class__):
+        if not isinstance(other, self.__class__):
             raise NotImplementedError(f"Cannot compute gcd: {other} must be a polynomial.")
+        if not self:
+            if not other:
+                return Poly([self.coeff[0]])
+            tmp = other / other.coeff[-1]
+            tmp.modulus = None
+            return tmp
         r0, r1 = other, self
         while r1:
             r0, r1 = r1, r0.divmod(r1)[1]
-        return r0
+        r0.modulus = None
+        return r0 / r0.coeff[-1]
 
     def egcd(self, other: "Poly") -> ("Poly", "Poly", "Poly"):
         """Perform the extended Euclidean agorithm for polynomials. Returns gcd, x, y such that other * x + self * y = gcd."""
         zero, one, ring = self._guess_ring()
-        if isinstance(other, list):
-            other = self.__class__(other, ring=ring)
-        elif not isinstance(other, self.__class__):
+        if not isinstance(other, self.__class__):
             raise NotImplementedError(f"Cannot perform egcd: {other} must be a polynomial.")
+        if not self:
+            if not other:
+                return Poly([zero]), Poly([zero]), Poly([zero])
+            return other / other.coeff[-1], Poly([zero]), one / other.coeff[-1]
         r0, r1 = other, self
-        x0, x1 = self.__class__([one], modulus=self.modulus), self.__class__([zero], modulus=self.modulus)
-        y0, y1 = self.__class__([zero], modulus=self.modulus), self.__class__([one], modulus=self.modulus)
+        x0, x1 = self.__class__([one]), self.__class__([zero])
+        y0, y1 = self.__class__([zero]), self.__class__([one])
         while r1:
             q, r = r0.divmod(r1)
             r0, r1 = r1, r
-            x0, x1 = x1, x0 - q * x1
-            y0, y1 = y1, y0 - q * y1
-        return r0, x0, y0
+            x0, x1 = x1, x0 - x1 * q
+            y0, y1 = y1, y0 - y1 * q
+        lcr0 = r0.coeff[-1]
+        return r0 / lcr0, x0 / lcr0, y0 / lcr0
 
     def derivative(self) -> "Poly":
         "Returns the formal derivative."
@@ -375,39 +391,147 @@ class Poly:
             tmp = [ j * self.coeff[j] for j in range(1, l) ]
         return self.__class__(tmp, modulus=self.modulus)
 
-    def rabin_test(self) -> bool:
-        """Determine if a polynomial is irreducible over a Galois field using a Rabin test."""
-        ring = self._guess_ring()[-1]
-        if not ring:
-            raise ValueError("The polynomial does not seem to be over a known finite field.")
-        try:
-            q = ring.n  # Zmod
-            isfield = is_prime(q)
-        except:
-            q = ring.order  # GF2
-            isfield = True
-        if not isfield:
-            raise ValueError("The polynomial does not seem to be over a finite field.")
-        #n = self.degree()
-        #x = self.__class__([0, 1], ring=ring, modulus = self.coeff)  # x
-        #for p in factorint(n).keys():
-        #    g = self.gcd(x**(q**(n//p)) - x)  # gcd with x^{q/n} - x
-        #    if g.degree():
-        #        return False
-        #g = x**(q**n) - x
-        #return not(g)
-        x = Poly([0, 1], ring = ring)  # x
-        xq = Poly(q * [0] + [1], ring = ring, modulus = self.coeff)  # x^q modulo self to keep the polynomials small
-        for k in range(1, self.degree() // 2 + 1):
+    def square_free_factors(self) -> dict:
+        "Determine the square free factors of a polynomial over a Galois field."
+        _, zero, _, q, p = self._guess_field()
+        pr = q // p # x^pr will be the p'th root of x
+        if self.degree() < 1:
+            return {}
+        if self.degree() == 1:
+            return { self  / self.coeff[-1]: 1}
+        factors = {}
+        w = self / self.coeff[-1]
+        c = w.gcd(w.derivative())
+        w = w // c
+
+        # first we find all factors which are not a power of p 
+        i = 1
+        while w.degree() > 0:
+            y = w.gcd(c)
+            fac = w // y
+            if fac.degree() > 0:
+                factors[fac] = i
+            w = y
+            c //= y
+            i += 1
+        # c is now a power of p
+        if c.degree() > 0:
+            # compute the p'th root of c
+            d = c.degree() // p
+            root = [ zero ] * (d + 1)
+            while d >= 0:
+                root[d] = c.coeff[d * p]**pr
+                d -= 1
+            #assert Poly(root)**p == c, root
+            for fac, mult in Poly(root).square_free_factors().items():
+                factors[fac] = p * mult
+        return factors
+
+    def is_square_free(self) -> bool:
+        "Test if the polynomial over a Galois field is square free."
+        multiplicities = self.square_free_factors().values()
+        return len(multiplicities) == 1 and multiplicities[0] == 1
+
+    def distinct_degree_factors(self, test:bool = False) -> bool|dict:
+        "Factors the polynomial over a Galois field into distinct products of irredicible factors of the same degree."
+        _, zero, one, q, _ = self._guess_field()
+        if self.degree() < 1:
+            raise ValueError("The polynomial must be non-constant.")
+        factors = {}
+        w = self
+        x = Poly([zero, one])  # x
+        xq = Poly(q * [zero] + [one], modulus = self.coeff)  # x^q modulo self to keep the polynomials small
+        k = 1
+        while k <= w.degree()//2:
             if k == 1:
                 b = xq
             else:
                 b **= q  # we compute x^{q^k} mod a recursively
-            g = self.gcd(b - x)  # gcd with x^{q^k} - x
+            g = w.gcd(b - x)  # gcd with x^{q^k} - x
             if g.degree():
-                return False
-        return True
+                if test:
+                    return False
+                factors[k] = g
+                w = w // factors[k]
+            k += 1
+        if test:
+            return True
+        if w.degree() > 0:
+            factors[w.degree()] = w
+        return factors
 
+    def rabin_test(self) -> bool:
+        "Determine if a polynomial is irreducible over a Galois field using a Rabin test."
+        return self.distinct_degree_factors(test = True)
+
+    def is_irreducible(self) -> bool:
+        "Test if the polynomial over a Galois field is irreducibel."
+        if self.degree() == 0:
+            return False
+        return self.distinct_degree_factors(test = True)
+
+    def equal_degree_factors(self ,k:int) -> list:
+        "Factors a square free product of irreducibel polynomials of degree `k` over a Galois field."
+        field, zero, one, q, p = self._guess_field()
+        if hasattr(field, "degree"):
+            degree= field.degree
+        else:
+            degree = 1
+        d = self.degree()
+        if not isinstance(k, int) or k < 1 :
+            raise ValueError(f"The degree {k} must be a positive integer.")
+        if d < k or d % k:
+            raise ValueError(f"The polynomial is no product of irreducibel polynomials of degree {k}.")
+        r = d // k
+        qk = (q**k - 1) // 2
+        poly = [0] * k + [1]
+ 
+        factors = [ self ]
+        while len(factors) < r:
+            g = Poly([randint(0,q-1) for _ in range(d+1)], ring = field, modulus = self)
+            if p == 2: # Gathen-Shoup
+                h = g
+                for i in range(1,k):
+                    g += h**q
+                if degree > 1:
+                    h = g
+                    for i in range(1,degree):
+                        g += g**2
+            else:  # Cantor–Zassenhaus
+                g **= qk
+            factors2 = []  # list for the next round (to avoid the the new factors are tested a second time with the same g)
+            for fac in factors:
+                if fac.degree() == k:  # this one is already irreducible
+                    factors2.append(fac)
+                    continue
+                gg = fac.gcd(g)
+                if gg.degree() == 0 or gg.degree() == fac.degree():  # trivial factor
+                    factors2.append(fac)
+                    continue
+                gg.modulus = None
+                factors2.append(gg)
+                factors2.append(fac // gg)
+            factors = factors2
+        return factors
+
+    def factor(self) -> dict:
+        "Factors a polynomials over a Galois field."
+        if self.degree() < 1:
+            return {self: 1}
+        c = Poly([self.coeff[-1]])
+        one = c**0
+        if c != one:
+            factors = {c: 1}
+        else:
+            factors = {}
+        for fac1, m in self.square_free_factors().items():
+            for k, fac2 in fac1.distinct_degree_factors().items():
+                if k == fac2.degree(): # irreducible
+                    factors[fac2] = m
+                else:
+                    for fac3 in fac2.equal_degree_factors(k):
+                        factors[fac3] = m
+        return dict(sorted(factors.items(), key=lambda item: item[0].degree()))
 
 def lagrange_interpolation(x_coordinates:list, y_coordinates:list) -> "Poly":
     "Compute the Lagrange interpolation polynomial from a given list of x and y values."
@@ -416,7 +540,7 @@ def lagrange_interpolation(x_coordinates:list, y_coordinates:list) -> "Poly":
         raise ValueError('List of x and y values must be nonempty of equal length.')
     if l != len(set(x_coordinates)):
         raise ValueError('List of x values must not contain duplicate values.')
-    # Guess zero and one
+    # guess zero and one
     zero = 0 * x_coordinates[0]
     one = zero**0
 
